<?php

namespace EventPortal\Domain\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Form\ValueObjects\ValidationData;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;

class UpdateValidations
{
    public function execute(Collection $validations, Question $dbQuestion, bool $isRequired = false): void
    {
        self::deleteUnusedValidations($validations, $dbQuestion);
        if (self::isRequiredValidationNeeded($isRequired, $dbQuestion)) {
            $validations->prepend(new ValidationData([
                'rule' => 'required',
                'message' => 'This field is required',
            ]));
        }

        $validations->each(static function (ValidationData $validation) use ($dbQuestion): void {
            QuestionValidation::updateOrCreate(
                ['id' => $validation->id],
                Arr::add($validation->toArray(), 'question_id', $dbQuestion->id)
            );
        });
    }

    private static function deleteUnusedValidations(Collection $validations, Question $question): void
    {
        QuestionValidation::query()
            ->whereNotIn('id', $validations->pluck('id'))
            ->forQuestion($question)
            ->delete();
    }

    private static function isRequiredValidationNeeded(bool $isRequired, Question $dbQuestion): bool
    {
        return $isRequired
            && QuestionValidation::forQuestion($dbQuestion)->isRequired()->exists() === false;
    }
}
