<?php

namespace EventPortal\Domain\Util;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Support\Str;

class RelationshipType
{
    /** @var \Illuminate\Database\Eloquent\Model */
    private $model;

    /** @var string */
    private $method;

    /** @var string */
    private $type;

    public function __construct(Model $model, string $method)
    {
        $this->model = $model;
        $this->method = $method;

        $this->type = $this->determineType();
    }

    public static function make(Model $model, string $method): self
    {
        return new self($model, $method);
    }

    public function typeIs(string $type): bool
    {
        return $this->type === $type;
    }

    public function getType(): string
    {
        return $this->type;
    }

    public function isBelongsToMany(): bool
    {
        return $this->typeIs(BelongsToMany::class);
    }

    private function determineType(): string
    {
        $className = get_class($this->model);

        if (Str::contains($this->method, '.') === false) {
            return get_class((new $className)->{$this->method}());
        }

        return $this->determineNestedType($className);
    }

    /**
     * Allows us to determine the relationship type of nested relationship
     * using dot notation.
     */
    private function determineNestedType(string $className): string
    {
        $methods = explode('.', $this->method);

        foreach ($methods as $key => $value) {
            if ($key === 0) {
                $related = (new $className)->{$value}()->getRelated();
            } else {
                if ($key === count($methods) - 1) {
                    $related = $related->{$value}();
                } else {
                    $related = $related->{$value}()->getRelated();
                }
            }
        }

        return get_class($related);
    }
}
