<?php

namespace EventPortal\Tests\Unit\EventDelegate\Actions;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Actions\AssociateToEvent;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class AssociateToEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function associate_delegates_to_event()
    {
        $event = factory(Event::class)->create();
        Pathway::generate($event, Pathway::DEFAULT, 'The default pathway');
        [$delegateA, $delegateB, $delegateC] = factory(Delegate::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $delegateA->id,
            $delegateC->id,
        ]);

        $this->assertEventHasDelegates($event, [$delegateA, $delegateC]);
        $this->assertEventMissingDelegate($event, $delegateB);
    }

    /** @test **/
    public function does_not_associate_more_than_once()
    {
        $event = factory(Event::class)->create();
        Pathway::generate($event, Pathway::DEFAULT, 'The default pathway');

        $delegate = factory(Delegate::class)->create();

        (new AssociateToEvent)->execute($event, [$delegate->id]);
        (new AssociateToEvent)->execute($event, [$delegate->id]);

        $this->assertCount(1, EventDelegate::all());
    }

    /** @test **/
    public function associate_delegates_to_event_with_default_pathway()
    {
        $event = factory(Event::class)->create();
        $pathway = Pathway::generate($event, Pathway::DEFAULT, 'The default pathway');
        [$delegateA, $delegateB, $delegateC] = factory(Delegate::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $delegateA->id,
            $delegateC->id,
        ]);

        $eventDelegateA = EventDelegate::forEventAndDelegate($event, $delegateA)->first();
        $eventDelegateB = EventDelegate::forEventAndDelegate($event, $delegateB)->first();
        $eventDelegateC = EventDelegate::forEventAndDelegate($event, $delegateC)->first();

        $this->assertSameModel($pathway, $eventDelegateA->pathway);
        $this->assertSameModel($pathway, $eventDelegateC->pathway);
        $this->assertNull($eventDelegateB);
    }
}
